@@ -11,8 +11,7 @@ const Me = ExtensionUtils.getCurrentExtension();
 const { WindowPicker, WindowPickerToggle } = Me.imports.windowPicker;
 const { WorkspaceIndicator } = Me.imports.workspaceIndicator;
 
-const Gettext = imports.gettext.domain(Me.metadata['gettext-domain']);
-const _ = Gettext.gettext;
+const _ = ExtensionUtils.gettext;
 
 const ICON_TEXTURE_SIZE = 24;
 const DND_ACTIVATE_TIMEOUT = 500;
@@ -23,34 +22,10 @@ const GroupingMode = {
     ALWAYS: 2,
 };
 
-
-function _minimizeOrActivateWindow(window) {
-    let focusWindow = global.display.focus_window;
-    if (focusWindow === window ||
-        focusWindow && focusWindow.get_transient_for() === window)
-        window.minimize();
-    else
-        window.activate(global.get_current_time());
-}
-
-function _openMenu(menu) {
-    menu.open();
-
-    let event = Clutter.get_current_event();
-    if (event && event.type() === Clutter.EventType.KEY_RELEASE)
-        menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
-}
-
-function _onMenuStateChanged(menu, isOpen) {
-    if (isOpen)
-        return;
-
-    let [x, y] = global.get_pointer();
-    let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
-    if (Me.stateObj.someWindowListContains(actor))
-        actor.sync_hover();
-}
-
+/**
+ * @param {Shell.App} app - an app
+ * @returns {number} - the smallest stable sequence of the app's windows
+ */
 function _getAppStableSequence(app) {
     let windows = app.get_windows().filter(w => !w.skip_taskbar);
     return windows.reduce((prev, cur) => {
@@ -58,7 +33,6 @@ function _getAppStableSequence(app) {
     }, Infinity);
 }
 
-
 class WindowContextMenu extends PopupMenu.PopupMenu {
     constructor(source, metaWindow) {
         super(source, 0.5, St.Side.BOTTOM);
@@ -132,17 +106,20 @@ class WindowContextMenu extends PopupMenu.PopupMenu {
     }
 }
 
-const WindowTitle = GObject.registerClass(
 class WindowTitle extends St.BoxLayout {
-    _init(metaWindow) {
-        this._metaWindow = metaWindow;
+    static {
+        GObject.registerClass(this);
+    }
 
-        super._init({
+    constructor(metaWindow) {
+        super({
             style_class: 'window-button-box',
             x_expand: true,
             y_expand: true,
         });
 
+        this._metaWindow = metaWindow;
+
         this._icon = new St.Bin({ style_class: 'window-button-icon' });
         this.add(this._icon);
         this.label_actor = new St.Label({ y_align: Clutter.ActorAlign.CENTER });
@@ -201,30 +178,33 @@ class WindowTitle extends St.BoxLayout {
         this._metaWindow.disconnect(this._notifyWmClass);
         this._metaWindow.disconnect(this._notifyAppId);
     }
-});
+}
 
+class BaseButton extends St.Button {
+    static {
+        GObject.registerClass({
+            GTypeFlags: GObject.TypeFlags.ABSTRACT,
+            Properties: {
+                'ignore-workspace': GObject.ParamSpec.boolean(
+                    'ignore-workspace', 'ignore-workspace', 'ignore-workspace',
+                    GObject.ParamFlags.READWRITE,
+                    false),
+            },
+        }, this);
+    }
 
-const BaseButton = GObject.registerClass({
-    GTypeFlags: GObject.TypeFlags.ABSTRACT,
-    Properties: {
-        'ignore-workspace': GObject.ParamSpec.boolean(
-            'ignore-workspace', 'ignore-workspace', 'ignore-workspace',
-            GObject.ParamFlags.READWRITE,
-            false),
-    },
-}, class BaseButton extends St.Button {
-    _init(perMonitor, monitorIndex) {
-        this._perMonitor = perMonitor;
-        this._monitorIndex = monitorIndex;
-        this._ignoreWorkspace = false;
-
-        super._init({
+    constructor(perMonitor, monitorIndex) {
+        super({
             style_class: 'window-button',
             can_focus: true,
             x_expand: true,
             button_mask: St.ButtonMask.ONE | St.ButtonMask.THREE,
         });
 
+        this._perMonitor = perMonitor;
+        this._monitorIndex = monitorIndex;
+        this._ignoreWorkspace = false;
+
         this.connect('notify::allocation',
             this._updateIconGeometry.bind(this));
         this.connect('clicked', this._onClicked.bind(this));
@@ -282,10 +262,37 @@ const BaseButton = GObject.registerClass({
         return true;
     }
 
+    _openMenu(menu) {
+        menu.open();
+
+        let event = Clutter.get_current_event();
+        if (event && event.type() === Clutter.EventType.KEY_RELEASE)
+            menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
+    }
+
+    _minimizeOrActivateWindow(window) {
+        let focusWindow = global.display.focus_window;
+        if (focusWindow === window ||
+            focusWindow && focusWindow.get_transient_for() === window)
+            window.minimize();
+        else
+            window.activate(global.get_current_time());
+    }
+
+    _onMenuStateChanged(menu, isOpen) {
+        if (isOpen)
+            return;
+
+        let [x, y] = global.get_pointer();
+        let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
+        if (Me.stateObj.someWindowListContains(actor))
+            actor.sync_hover();
+    }
+
     _onPopupMenu(_actor) {
         if (!this._canOpenPopupMenu() || this._contextMenu.isOpen)
             return;
-        _openMenu(this._contextMenu);
+        this._openMenu(this._contextMenu);
     }
 
     _isFocused() {
@@ -343,13 +350,15 @@ const BaseButton = GObject.registerClass({
             global.display.disconnect(this._windowLeftMonitorId);
         this._windowLeftMonitorId = 0;
     }
-});
+}
 
-
-const WindowButton = GObject.registerClass(
 class WindowButton extends BaseButton {
-    _init(metaWindow, perMonitor, monitorIndex) {
-        super._init(perMonitor, monitorIndex);
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(metaWindow, perMonitor, monitorIndex) {
+        super(perMonitor, monitorIndex);
 
         this.metaWindow = metaWindow;
         this._skipTaskbarId = metaWindow.connect('notify::skip-taskbar', () => {
@@ -362,7 +371,8 @@ class WindowButton extends BaseButton {
         this.label_actor = this._windowTitle.label_actor;
 
         this._contextMenu = new WindowContextMenu(this, this.metaWindow);
-        this._contextMenu.connect('open-state-changed', _onMenuStateChanged);
+        this._contextMenu.connect('open-state-changed',
+            this._onMenuStateChanged.bind(this));
         this._contextMenu.actor.hide();
         this._contextMenuManager.addMenu(this._contextMenu);
         Main.uiGroup.add_actor(this._contextMenu.actor);
@@ -382,9 +392,9 @@ class WindowButton extends BaseButton {
         }
 
         if (button === 1)
-            _minimizeOrActivateWindow(this.metaWindow);
+            this._minimizeOrActivateWindow(this.metaWindow);
         else
-            _openMenu(this._contextMenu);
+            this._openMenu(this._contextMenu);
     }
 
     _isFocused() {
@@ -420,8 +430,7 @@ class WindowButton extends BaseButton {
         global.display.disconnect(this._notifyFocusId);
         this._contextMenu.destroy();
     }
-});
-
+}
 
 class AppContextMenu extends PopupMenu.PopupMenu {
     constructor(source) {
@@ -481,10 +490,13 @@ class AppContextMenu extends PopupMenu.PopupMenu {
     }
 }
 
-const AppButton = GObject.registerClass(
 class AppButton extends BaseButton {
-    _init(app, perMonitor, monitorIndex) {
-        super._init(perMonitor, monitorIndex);
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(app, perMonitor, monitorIndex) {
+        super(perMonitor, monitorIndex);
 
         this.app = app;
         this._updateVisibility();
@@ -518,14 +530,16 @@ class AppButton extends BaseButton {
 
         this._menuManager = new PopupMenu.PopupMenuManager(this);
         this._menu = new PopupMenu.PopupMenu(this, 0.5, St.Side.BOTTOM);
-        this._menu.connect('open-state-changed', _onMenuStateChanged);
+        this._menu.connect('open-state-changed',
+            this._onMenuStateChanged.bind(this));
         this._menu.actor.hide();
         this._menu.connect('activate', this._onMenuActivate.bind(this));
         this._menuManager.addMenu(this._menu);
         Main.uiGroup.add_actor(this._menu.actor);
 
         this._appContextMenu = new AppContextMenu(this);
-        this._appContextMenu.connect('open-state-changed', _onMenuStateChanged);
+        this._appContextMenu.connect('open-state-changed',
+            this._onMenuStateChanged.bind(this));
         this._appContextMenu.actor.hide();
         Main.uiGroup.add_actor(this._appContextMenu.actor);
 
@@ -592,7 +606,7 @@ class AppButton extends BaseButton {
                 this._singleWindowTitle.child = this._windowTitle;
                 this._windowContextMenu = new WindowContextMenu(this, this.metaWindow);
                 this._windowContextMenu.connect(
-                    'open-state-changed', _onMenuStateChanged);
+                    'open-state-changed', this._onMenuStateChanged.bind(this));
                 Main.uiGroup.add_actor(this._windowContextMenu.actor);
                 this._windowContextMenu.actor.hide();
                 this._contextMenuManager.addMenu(this._windowContextMenu);
@@ -631,7 +645,7 @@ class AppButton extends BaseButton {
             if (windows.length === 1) {
                 if (contextMenuWasOpen)
                     return;
-                _minimizeOrActivateWindow(windows[0]);
+                this._minimizeOrActivateWindow(windows[0]);
             } else {
                 this._menu.removeAll();
 
@@ -642,12 +656,12 @@ class AppButton extends BaseButton {
                     item._window = windows[i];
                     this._menu.addMenuItem(item);
                 }
-                _openMenu(this._menu);
+                this._openMenu(this._menu);
             }
         } else {
             if (contextMenuWasOpen)
                 return;
-            _openMenu(this._contextMenu);
+            this._openMenu(this._contextMenu);
         }
     }
 
@@ -666,16 +680,15 @@ class AppButton extends BaseButton {
         this.app.disconnect(this._windowsChangedId);
         this._menu.destroy();
     }
-});
+}
 
-
-const WindowList = GObject.registerClass(
 class WindowList extends St.Widget {
-    _init(perMonitor, monitor) {
-        this._perMonitor = perMonitor;
-        this._monitor = monitor;
+    static {
+        GObject.registerClass(this);
+    }
 
-        super._init({
+    constructor(perMonitor, monitor) {
+        super({
             name: 'panel',
             style_class: 'bottom-panel solid',
             reactive: true,
@@ -684,6 +697,9 @@ class WindowList extends St.Widget {
         });
         this.connect('destroy', this._onDestroy.bind(this));
 
+        this._perMonitor = perMonitor;
+        this._monitor = monitor;
+
         let box = new St.BoxLayout({ x_expand: true, y_expand: true });
         this.add_actor(box);
 
@@ -773,13 +789,16 @@ class WindowList extends St.Widget {
             this._updateKeyboardAnchor();
         });
 
-        this._overviewHidingId = Main.overview.connect('hiding', () => {
+        this._overviewHidingId = Main.overview.connect('hidden', () => {
             this.visible = !Main.layoutManager.primaryMonitor.inFullscreen;
             this._updateKeyboardAnchor();
         });
 
         this._fullscreenChangedId =
             global.display.connect('in-fullscreen-changed', () => {
+                // Work-around for initial change from unknown to !fullscreen
+                if (Main.overview.visible)
+                    this.hide();
                 this._updateKeyboardAnchor();
             });
 
@@ -1075,7 +1094,7 @@ class WindowList extends St.Widget {
         for (let i = 0; i < windows.length; i++)
             windows[i].metaWindow.set_icon_geometry(null);
     }
-});
+}
 
 class Extension {
     constructor() {
@@ -1144,6 +1163,9 @@ class Extension {
     }
 }
 
+/**
+ * @returns {Extension} - the extension's state object
+ */
 function init() {
     return new Extension();
 }
